"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useField = exports.useForm = exports.FormConsumer = exports.FormProvider = exports.FormContext = exports.rules = void 0;
const react_1 = require("react");
const lodash_1 = require("lodash");
exports.rules = {
    required: (message = 'This field is required') => (value) => {
        if (!value || !value.toString().length) {
            return message;
        }
    },
    email: (message = 'Email address is invalid') => (value) => {
        const isEmail = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
        if (!isEmail.test(String(value).toLowerCase())) {
            return message;
        }
    },
    matches: (regex, message) => (value) => {
        if (regex.test(String(value))) {
            return message;
        }
    },
    matchesValue: (matcher, message) => (value, values) => {
        if ((typeof matcher === 'string' ? values[matcher] : matcher(values)) !== value) {
            return message;
        }
    },
    requiredIf: (predicate, message) => (value, values) => {
        if ((!value || !value.toString().length) && predicate(values)) {
            return message;
        }
    },
};
exports.FormContext = (0, react_1.createContext)({});
exports.FormProvider = exports.FormContext.Provider;
exports.FormConsumer = exports.FormContext.Consumer;
const defaultFormOptions = {
    validateOnBlur: true,
};
const useForm = ({ onSubmit, values: initialValues, options = defaultFormOptions, }) => {
    const rules = (0, react_1.useRef)({});
    const [values, setValues] = (0, react_1.useState)(initialValues || {});
    const [blurred, setBlurred] = (0, react_1.useState)({});
    const [errors, setErrors] = (0, react_1.useState)([]);
    const [isSubmitting, setIsSubmiting] = (0, react_1.useState)(false);
    const [hasSubmitted, setHasSubmitted] = (0, react_1.useState)(false);
    return {
        rules,
        values,
        errors,
        blurred,
        isSubmitting,
        hasSubmitted,
        options,
        setValues,
        setErrors,
        setBlurred,
        hasBlurred: (name) => {
            if (blurred[name]) {
                return;
            }
            setBlurred(Object.assign(Object.assign({}, blurred), { [name]: true }));
        },
        updateValue: (name, value) => {
            setValues((currentValues) => {
                return (0, lodash_1.set)(Object.assign({}, currentValues), name, value);
            });
        },
        handleSubmit: (event) => {
            event.preventDefault();
            setHasSubmitted(true);
            const errors = validate({ values, rules });
            setErrors(errors);
            if (!errors.length) {
                setIsSubmiting(true);
                return Promise.resolve(onSubmit(values)).finally(() => {
                    setIsSubmiting(false);
                });
            }
            setBlurred(Object.keys(rules.current).reduce((keys, name) => {
                return Object.assign(Object.assign({}, keys), { [name]: true });
            }, {}));
        },
    };
};
exports.useForm = useForm;
const validate = ({ values, rules, blurred }) => {
    return Object.keys(rules.current)
        .filter((field) => {
        if (blurred)
            return blurred[field];
        return true;
    })
        .map((field) => rules.current[field].map((rule) => {
        const error = rule((0, lodash_1.get)(values, field) || '', values);
        if (!error) {
            return false;
        }
        return {
            name: field,
            message: error,
        };
    }))
        .reduce((errors, row) => [...errors, ...row], [])
        .filter(Boolean);
};
const useField = ({ name, value: defaultValue, onChange = () => { }, rules: fieldRules }) => {
    const { errors, values, setErrors, hasBlurred, updateValue, blurred, rules, options } = (0, react_1.useContext)(exports.FormContext);
    (0, react_1.useEffect)(() => {
        rules.current[name] = fieldRules || [];
        updateValue(name, defaultValue);
        return () => {
            delete rules.current[name];
        };
    }, [name, defaultValue, fieldRules]);
    const fieldErrors = errors.filter((error) => error.name === name).map((error) => error.message);
    const validation = {
        values,
        rules,
        blurred,
        errors,
    };
    const handleChange = (value) => {
        if (fieldErrors.length || blurred[name]) {
            const newValues = Object.assign({}, values);
            (0, lodash_1.set)(newValues, name, value);
            setErrors(() => validate(Object.assign(Object.assign({}, validation), { values: newValues, blurred: Object.assign(Object.assign({}, blurred), { [name]: true }) })));
        }
        onChange(value);
        updateValue(name, value);
    };
    const handleBlur = () => {
        hasBlurred(name);
        options.validateOnBlur &&
            setErrors(() => validate(Object.assign(Object.assign({}, validation), { blurred: Object.assign(Object.assign({}, blurred), { [name]: true }) })));
    };
    return {
        handleBlur,
        handleChange,
        value: (0, lodash_1.get)(values, name),
        values,
        errors: fieldErrors.length ? fieldErrors : null,
    };
};
exports.useField = useField;
//# sourceMappingURL=useForm.js.map